@@ -2,6 +2,7 @@
  * IIIMF-SKK, Japanese Language Engine for 
  *                        IIIMF (Internet/Intranet Input Method Framework)
  * 
+ * Copuright (C) 2003 Motonobu Ichimura <famao@momonga-linux.org>
  * Copyright (C) 2002 Motonobu Ichimura <famao@kondara.org>
  *
  * All rights reserved.
@@ -33,7 +34,7 @@
  *
  */
 
-/* $Id: adddict.c,v 1.4 2002/11/09 16:56:47 famao Exp $ */
+/* $Id: adddict.c,v 1.1.1.1.2.6 2003/02/12 22:18:09 famao Exp $ */
 
 /* vi:set ts=4 sw=4: */
 
@@ -51,6 +52,10 @@
 #include <sys/wait.h>
 #include <errno.h>
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include "xaux_common.h"
 #include "xaux_ext_common.h"
 #include "skk_xaux.h"
@@ -58,7 +63,7 @@
 #define SKK_AUX_DIR "/usr/lib/im/locale/ja/skk/xaux"
 
 xaux_class_t xaux_class = {
-	"org.kondara.skk.AddDictAux",1,
+	"org.momonga-linux.skk.AddDictAux",1,
 	NULL, (Window) 0, (Window) 0,
 	(Atom) 0, (Atom) 0, (Atom) 0,
 	{(Atom) 0,}, 0, {(Atom) 0, },
@@ -77,10 +82,52 @@ static int im_ = 0;
 static int ic_ = 0;
 /* static int wait_process = 0; */
 
+/* ProtoType */
+/* Error Handling */
 static int adddict_x_error (Display *d, XErrorEvent *error);
+/* Event */
+static GdkFilterReturn ext_event_handler (GdkXEvent *gdk_xevent, GdkEvent *event, gpointer user_data);
+static void input_func (gpointer data, gint source, GdkInputCondition condition);
+
+static void parse_line_from_child (gchar *str);
+static void reply_initialize (void);
+static void execute (gchar *str);
+static gchar *escape_str (const gchar *str);
 
 void skk_aux_adddict_send_engine (gint int_count, gint *integers, gint str_count, gchar **strings);
 
+static gchar *
+escape_str (const gchar *str)
+{
+	/* Dirty Hack */
+	gchar *q;
+	const gchar *p = str;
+	gchar *ret = NULL;
+	guint esc = 0;
+	if (!str)
+		return NULL;
+	while (*p != '\000') {
+		if (*p == '\'' || *p == '"' )
+			esc++;
+		p++;
+	}
+
+	if (!esc)
+		return g_strdup (str);
+
+	ret = g_new (gchar, strlen (str) + esc + 1);
+	p = str;
+	q = ret;
+
+	while (*p != '\000') {
+		if (*p == '\'' || *p == '"')
+			*q++ = '\\';
+		*q++ = *p++;
+	}
+	*q = '\000';
+	return ret;
+}
+
 static int
 adddict_x_error (Display *d, XErrorEvent *error)
 {
@@ -265,7 +312,15 @@ execute (gchar *str)
 	pid = fork ();
 	if (pid == 0) {
 		close (fds[0]);
-		cmd = g_strdup_printf ("%s%s %s", SKK_AUX_DIR, "/AddDictAuxHelper", str);
+		/* FIXME */
+		if (!strchr (str, '\'')) {
+		    cmd = g_strdup_printf ("%s%s '%s'", SKK_AUX_DIR, "/AddDictAuxHelper", str);
+		} else {
+			gchar *escape = escape_str (str);
+			printf ("escape %s\n", escape);
+			cmd = g_strdup_printf ("%s%s %s", SKK_AUX_DIR, "/AddDictAuxHelper", escape);
+			g_free (escape);
+		}
 		/* remove event filter */
 		gdk_window_remove_filter (window->window, ext_event_handler, NULL);
 		fd = popen (cmd, "r");
@@ -349,9 +404,9 @@ main (int argc, char *argv[])
 {
 	/* GtkWidget *window; */
 #ifdef HAVE_SETENV
-	setenv ("LC_ALL","ja_JP.eucJP",TRUE);
+	setenv ("LC_ALL", JA_LOCALE ,TRUE);
 #elif HAVE_PUTENV
-	putenv ("LC_ALL=ja_JP.eucJP");
+	putenv ("LC_ALL=" JA_LOCALE);
 #endif
 	gtk_set_locale ();
 	gtk_init (&argc,&argv);
