@@ -33,7 +33,7 @@
  *
  */
 
-/* $Id: codetable.c,v 1.7 2002/11/09 16:56:47 famao Exp $ */
+/* $Id: codetable.c,v 1.1.1.1.2.6 2003/02/13 10:21:22 famao Exp $ */
 
 /* vi:set ts=4 sw=4: */
 
@@ -474,7 +474,7 @@ skk_aux_code_new (void)
 	gtk_signal_connect (GTK_OBJECT (button), "clicked",
 			GTK_SIGNAL_FUNC (code_button_clicked), (gpointer) &default_codetable[5]);
 	
-	button = gtk_button_new_with_label (_("Kanji"));
+	button = gtk_button_new_with_label (_("Box Drawing"));
 	gtk_widget_show (button);
 	gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 0);
 	gtk_signal_connect (GTK_OBJECT (button), "clicked",
@@ -547,9 +547,9 @@ main (int argc, char *argv[])
 	GtkWidget *toplevel;
 	/* force to EUC-JP */
 #ifdef HAVE_SETENV
-	setenv ("LC_ALL","ja_JP.EUC-JP",TRUE);
+	setenv ("LC_ALL",JA_LOCALE ,TRUE);
 #elif HAVE_PUTENV
-	putenv ("LC_ALL=ja_JP.EUC-JP");
+	putenv ("LC_ALL=" JA_LOCALE);
 #endif
 	gtk_set_locale ();
 
