@@ -1,6 +1,7 @@
 /*
  * LibSKK, a tiny Library to emulate SKK (Simple Kana Kanji Conversion)
  * 
+ * Copyright (C) 2003 Motonobu Ichimura <famao@momonga-linux.org>
  * Copyright (C) 2002 Motonobu Ichimura <famao@kondara.org>
  *
  * All rights reserved.
@@ -32,7 +33,7 @@
  *
  */
 
-/* $Id: skkbuffer.c,v 1.21 2002/11/09 16:56:48 famao Exp $ */
+/* $Id: skkbuffer.c,v 1.14.2.38 2003/02/12 11:30:14 famao Exp $ */
 
 /* vi:set ts=4 sw=4: */
 
@@ -234,11 +235,15 @@ separate_string_by_cursor (SkkBuffer *buf, const gchar *string, gchar **beforep,
 		after += offset;
 		if (beforep) {
 			*beforep = g_strndup (string, offset);
+#if 0
 			g_message ("beforep %s", *beforep);
+#endif
 		}
 		if (afterp) {
 			*afterp = g_strdup (after);
+#if 0
 			g_message ("after %s", after);
+#endif
 		}
 		return TRUE;
 	}
@@ -248,9 +253,9 @@ separate_string_by_cursor (SkkBuffer *buf, const gchar *string, gchar **beforep,
 static void
 lookup_emit (SkkBuffer *buf)
 {
-	GList *tmp_list;
+	GSList *tmp_list;
 	SkkLookupFunc *func;
-	gchar *annotate[7] = { NULL, NULL, NULL, NULL, NULL, NULL, NULL};
+	gchar *annotation[7] = { NULL, NULL, NULL, NULL, NULL, NULL, NULL};
 	gchar *candidate[7] = { NULL, NULL, NULL, NULL, NULL, NULL, NULL};
 	gchar *label[7] = { "A:", "S:", "D:", "F:", "J:", "K:", "L:"};  /* TODO */
 	gint count = 0;
@@ -259,6 +264,7 @@ lookup_emit (SkkBuffer *buf)
 	restore = buf->candidate_count;
 	for (i = 0; i < 7; i++) {
 		candidate[i] = skk_buffer_get_cur_candidate (buf);
+		annotation[i] = skk_buffer_get_cur_annotation (buf);
 		if (candidate[i]) {
 			count++;
 		}
@@ -268,18 +274,18 @@ lookup_emit (SkkBuffer *buf)
 			break;
 		}
 	}
-	for (tmp_list = buf->lookup_listener; tmp_list; tmp_list = g_list_next (tmp_list)) {
+	for (tmp_list = buf->lookup_listener; tmp_list; tmp_list = g_slist_next (tmp_list)) {
 		func = (SkkLookupFunc *)tmp_list->data;
 		if (func) {
-			func->listener (buf, label, candidate, annotate, count, func->user_data);
+			func->listener (buf, label, candidate, annotation, count, func->user_data);
 		}
 	}
 	for (i = 0; i < 7; i++) {
 		/* Free */
 		if (candidate[i])
 			g_free (candidate[i]);
-		if (annotate[i])
-			g_free (annotate[i]);
+		if (annotation[i])
+			g_free (annotation[i]);
 	}
 	/* Restore */
 	buf->candidate_count = restore;
@@ -289,7 +295,7 @@ lookup_emit (SkkBuffer *buf)
 static void
 adddict_emit (SkkBuffer *buf)
 {
-	GList *tmp_list;
+	GSList *tmp_list;
 	SkkAddDictFunc *func;
 	gchar *query_str;
 	if (!buf)
@@ -299,7 +305,7 @@ adddict_emit (SkkBuffer *buf)
 	} else {
 		query_str = g_strdup (skkbuf_d (buf));
 	}
-	for (tmp_list = buf->adddict_listener; tmp_list; tmp_list = g_list_next (tmp_list)) {
+	for (tmp_list = buf->adddict_listener; tmp_list; tmp_list = g_slist_next (tmp_list)) {
 		func = (SkkAddDictFunc *)tmp_list->data;
 		if (func) {
 			func->listener (buf, query_str, func->user_data);
@@ -312,11 +318,11 @@ adddict_emit (SkkBuffer *buf)
 static void
 codetable_emit (SkkBuffer *buf)
 {
-	GList *tmp_list;
+	GSList *tmp_list;
 	SkkCodeTableFunc *func;
 	if (!buf)
 		return;
-	for (tmp_list = buf->codetable_listener; tmp_list; tmp_list = g_list_next (tmp_list)) {
+	for (tmp_list = buf->codetable_listener; tmp_list; tmp_list = g_slist_next (tmp_list)) {
 		func = (SkkCodeTableFunc *)tmp_list->data;
 		if (func) {
 			func->listener (buf, func->user_data);
@@ -328,7 +334,7 @@ codetable_emit (SkkBuffer *buf)
 static void
 preedit_emit (SkkBuffer *buf)
 {
-	GList* tmp_list;
+	GSList* tmp_list;
 	SkkBufFunc *func;
 	gchar *preedit;
 	if (!buf)
@@ -343,12 +349,14 @@ preedit_emit (SkkBuffer *buf)
 		if (skkbuf_cursor (buf)->caret == 0) {
 			skkbuf_cursor (buf)->revLen = skkbuf_plen (buf);
 		}
+#if 0
 		g_message ("revLen %d", skkbuf_cursor (buf)->revLen);
+#endif
 	} else {
 		skkbuf_cursor (buf)->revPos = 0;
 		skkbuf_cursor (buf)->revLen = 0;
 	}
-	for (tmp_list = buf->preedit_listener; tmp_list; tmp_list = g_list_next (tmp_list)) {
+	for (tmp_list = buf->preedit_listener; tmp_list; tmp_list = g_slist_next (tmp_list)) {
 		func = (SkkBufFunc *)tmp_list->data;
 		if (func) {
 			func->listener (buf, preedit, func->user_data);
@@ -363,11 +371,11 @@ preedit_emit (SkkBuffer *buf)
 static void
 result_emit (SkkBuffer *buf)
 {
-	GList *tmp_list;
+	GSList *tmp_list;
 	SkkBufFunc *func;
 	if (!buf)
 		return;
-	for (tmp_list = buf->result_listener; tmp_list; tmp_list = g_list_next (tmp_list)) {
+	for (tmp_list = buf->result_listener; tmp_list; tmp_list = g_slist_next (tmp_list)) {
 		func = (SkkBufFunc *)tmp_list->data;
 		if (func) {
 			func->listener (buf, skkbuf_r (buf), func->user_data);
@@ -504,7 +512,6 @@ query_num (SkkBuffer *buf)
 			/* found */
 			if (num) {
 				buf->candidate_list = skk_dict_item_merge_list (buf->candidate_list, num);
-				buf->candidate_max = g_list_length (buf->candidate_list);
 #if 0
 				for ( tmp = num , count = buf->candidate_count
 						; tmp ;
@@ -521,6 +528,7 @@ query_num (SkkBuffer *buf)
 				*/
 #endif
 			}
+			buf->candidate_max = g_list_length (buf->candidate_list);
 			g_free (ret);
 		} else {
 			tmp = g_list_nth (buf->candidate_list, buf->candidate_count);
@@ -528,6 +536,8 @@ query_num (SkkBuffer *buf)
 			tmp->data = skk_dict_item_new_with_value (ret, NULL);
 			g_free (ret);
 		}
+	} else {
+		buf->candidate_max = g_list_length (buf->candidate_list);
 	}
 	return;
 }
@@ -536,7 +546,7 @@ static void
 query_dict (SkkBuffer *buf)
 {
 	gchar *preedit;
-	static gint candidate_count = -1;
+	gint candidate_count = 0;
 	SkkCursor *cursor;
 	DEBUG_DO (printf ("skkbuffer.c : Entering query_dict\n"));
 	if (!buf)
@@ -560,9 +570,7 @@ query_dict (SkkBuffer *buf)
 				}
 			}
 		}
-		if (candidate_count == -1) {
-			candidate_count = skk_conf_get_num_by_name (skkbuf_conf (buf), "skk-candidate-count");
-		}
+		candidate_count = skk_conf_get_num_by_name (skkbuf_conf (buf), "skk-candidate-count");
 		preedit_emit (buf);
 		if (candidate_count == 0) {
 			skk_buffer_lookup_emit (buf);
@@ -2043,23 +2051,23 @@ skk_buffer_destroy (SkkBuffer *buf)
 	skk_conv_rule_destroy (buf->rule);
 	skk_cursor_destroy (skkbuf_cursor (buf));
 	if (buf->preedit_listener) {
-		skk_utils_list_free (buf->preedit_listener, TRUE, NULL, NULL);
+		skk_utils_slist_free (buf->preedit_listener, TRUE, NULL, NULL);
 		buf->preedit_listener = NULL;
 	}
 	if (buf->result_listener) {
-		skk_utils_list_free (buf->result_listener, TRUE, NULL, NULL);
+		skk_utils_slist_free (buf->result_listener, TRUE, NULL, NULL);
 		buf->result_listener = NULL;
 	}
 	if (buf->lookup_listener) {
-		skk_utils_list_free (buf->lookup_listener, TRUE, NULL, NULL);
+		skk_utils_slist_free (buf->lookup_listener, TRUE, NULL, NULL);
 		buf->lookup_listener = NULL;
 	}
 	if (buf->adddict_listener) {
-		skk_utils_list_free (buf->adddict_listener, TRUE, NULL, NULL);
+		skk_utils_slist_free (buf->adddict_listener, TRUE, NULL, NULL);
 		buf->adddict_listener = NULL;
 	}
 	if (buf->codetable_listener) {
-		skk_utils_list_free (buf->adddict_listener, TRUE, NULL, NULL);
+		skk_utils_slist_free (buf->adddict_listener, TRUE, NULL, NULL);
 		buf->codetable_listener = NULL;
 	}
 	g_free (buf);
@@ -2352,9 +2360,14 @@ skk_buffer_get_nth_candidate (SkkBuffer *buf, gint num)
 		return NULL;
 	cur_count = buf->candidate_count;
 	buf->candidate_count = num;
+	if (buf->candidate_max <= num)
+		return NULL;
 	if (buf->has_num) {
 		query_num (buf);
 	}
+	/* check again */
+	if (buf->candidate_max <= num)
+		return NULL;
 	candidate = skkbuf_cur_candidate (buf);
 	buf->candidate_count = cur_count;
 	if (!candidate)
@@ -2380,6 +2393,9 @@ skk_buffer_get_cur_candidate (SkkBuffer *buf)
 	if (buf->has_num) {
 		query_num (buf);
 	}
+	/* check again */
+	if (buf->candidate_count == buf->candidate_max)
+		return NULL;
 	candidate = skkbuf_cur_candidate (buf);
 	if (!candidate)
 		return NULL;
@@ -2390,6 +2406,22 @@ skk_buffer_get_cur_candidate (SkkBuffer *buf)
 	}
 }
 
+gchar *
+skk_buffer_get_cur_annotation (SkkBuffer *buf)
+{
+	gchar *annotation;
+	if (!buf)
+		return NULL;
+	if (!buf->candidate_list)
+		return NULL;
+	if (buf->candidate_count == buf->candidate_max)
+		return NULL;
+	annotation = skkbuf_cur_annotation (buf);
+	if (!annotation)
+		return NULL;
+	return strdup (annotation);
+}
+
 gchar *
 skk_buffer_get_next_candidate (SkkBuffer *buf) 
 {
@@ -2721,7 +2753,7 @@ skk_buffer_add_codetable_listener (SkkBuffer *buf, SkkCodeTableListener listener
 	func = g_new (SkkCodeTableFunc, 1);
 	func->listener = listener;
 	func->user_data = user_data;
-	buf->codetable_listener = g_list_append (buf->codetable_listener, func);
+	buf->codetable_listener = g_slist_append (buf->codetable_listener, func);
 	return;
 }
 
@@ -2736,7 +2768,7 @@ skk_buffer_add_lookup_listener (SkkBuffer *buf, SkkLookupListener listener, gpoi
 	func = g_new (SkkLookupFunc, 1);
 	func->listener = listener;
 	func->user_data = user_data;
-	buf->lookup_listener = g_list_append (buf->lookup_listener, func);
+	buf->lookup_listener = g_slist_append (buf->lookup_listener, func);
 	return;
 }
 
@@ -2751,7 +2783,7 @@ skk_buffer_add_adddict_listener (SkkBuffer *buf, SkkAddDictListener listener, gp
 	func = g_new (SkkAddDictFunc, 1);
 	func->listener = listener;
 	func->user_data = user_data;
-	buf->adddict_listener = g_list_append (buf->adddict_listener, func);
+	buf->adddict_listener = g_slist_append (buf->adddict_listener, func);
 	return;
 }
 
@@ -2767,7 +2799,7 @@ skk_buffer_add_preedit_listener (SkkBuffer *buf, SkkBufListener listener, gpoint
 	func = g_new (SkkBufFunc, 1);
 	func->listener = listener;
 	func->user_data = user_data;
-	buf->preedit_listener = g_list_append (buf->preedit_listener, func);
+	buf->preedit_listener = g_slist_append (buf->preedit_listener, func);
 	return;
 }
 
@@ -2782,7 +2814,7 @@ skk_buffer_add_result_listener (SkkBuffer *buf, SkkBufListener listener, gpointe
 	func = g_new (SkkBufFunc, 1);
 	func->listener = listener;
 	func->user_data = user_data;
-	buf->result_listener = g_list_append (buf->result_listener, func);
+	buf->result_listener = g_slist_append (buf->result_listener, func);
 	return;
 }
 
@@ -2827,7 +2859,7 @@ skk_buffer_add_dict (SkkBuffer *buf)
 	if (!skkbuf_d (buf))
 		return;
 	/* FIXME not handle in the case of buf->has_num > 0) */
-	/* rewind search pligin */
+	/* rewind search plugin */
 	skk_query_set_first (skkbuf_query (buf));
 	skk_query_set_with_type (skkbuf_query (buf), SKKQUERY_LOCAL);
 	direction = get_direction_word (buf);
@@ -2845,6 +2877,7 @@ skk_buffer_add_dict (SkkBuffer *buf)
 void
 skk_buffer_add_dict_with_value (SkkBuffer *buf, gchar *value)
 {
+	gchar *direction;
 	if (!buf)
 		return;
 	if (!value)
@@ -2854,9 +2887,12 @@ skk_buffer_add_dict_with_value (SkkBuffer *buf, gchar *value)
 	/* rewind search plugin */
 	skk_query_set_first (skkbuf_query (buf));
 	skk_query_set_with_type (skkbuf_query (buf), SKKQUERY_LOCAL);
-	skk_query_add (buf->query, skkbuf_d (buf), skkbuf_okurip (buf), value);
+	direction = get_direction_word (buf);
+	skk_query_add (buf->query, direction ? direction : skkbuf_d (buf), skkbuf_okurip (buf), value);
 	/* reset */
 	skk_query_set_first (skkbuf_query (buf));
+	if (direction)
+		g_free (direction);
 	return;
 }
 
