@@ -1,6 +1,7 @@
 /*
  * LibSKK, a tiny Library to emulate SKK (Simple Kana Kanji Conversion)
  * 
+ * Copyright (C) 2003 Motonobu Ichimura <famao@momonga-linux.org>
  * Copyright (C) 2002 Motonobu Ichimura <famao@kondara.org>
  *
  * All rights reserved.
@@ -32,7 +33,7 @@
  *
  */
 
-/* $Id: skkconf.c,v 1.4 2002/11/09 16:56:48 famao Exp $ */
+/* $Id: skkconf.c,v 1.3.2.52 2003/02/12 11:36:08 famao Exp $ */
 
 /* vi:set ts=4 sw=4: */
 
@@ -57,14 +58,17 @@ static void parse_item (SkkConf *conf, xmlDocPtr doc);
 /* Parse Rule */
 static void parse_rule_one (SkkConf *conf, xmlNodePtr node);
 static void parse_rule (SkkConf *conf, xmlDocPtr doc);
+static void parse_rule_delete (SkkConf *conf, xmlDocPtr doc);
 
 static SkkConfType find_type (gchar *str);
 static SkkConfItemType find_item_type (const gchar *str);
+static SkkConfItem *find_item (GList *list, SkkConfItemType type);
 
 /* Add Item to List */
 static void add_item_bool (SkkConf *conf, SkkConfItem *item);
 static void add_item_string (SkkConf *conf, SkkConfItem *item);
 static void add_item_num (SkkConf *conf, SkkConfItem *item);
+static void add_defaults (SkkConf* conf);
 
 /* Find Item */
 static gboolean find_item_func (gconstpointer p1, gconstpointer p2);
@@ -77,11 +81,18 @@ static void load_rc (SkkConf *conf, const gchar *filename);
 /* Event */
 static void add_rule_emit (SkkConf *conf, const gchar *key, const gchar *hira,
 		const gchar *kata, const gchar *append);
+static void delete_rule_emit (SkkConf *conf, const gchar *key);
 
-typedef struct _SkkConfRuleFunc SkkConfRuleFunc;
+typedef struct _SkkConfAddRuleFunc SkkConfAddRuleFunc;
+typedef struct _SkkConfDeleteRuleFunc SkkConfDeleteRuleFunc;
 
-struct _SkkConfRuleFunc {
-	SkkConfRuleListener listener;
+struct _SkkConfAddRuleFunc {
+	SkkConfAddRuleListener listener;
+	gpointer user_data;
+};
+
+struct _SkkConfDeleteRuleFunc {
+	SkkConfDeleteRuleListener listener;
 	gpointer user_data;
 };
 
@@ -89,13 +100,13 @@ static void
 add_rule_emit (SkkConf *conf, const gchar *key, const gchar *hira,
 		const gchar *kata, const gchar *append)
 {
-	GList *tmp_list;
-	SkkConfRuleFunc *func;
+	GSList *tmp_list;
+	SkkConfAddRuleFunc *func;
 	if (!conf)
 		return;
-	for (tmp_list = conf->conf_rule_listener;
-			tmp_list; tmp_list = g_list_next (tmp_list)) {
-		func = (SkkConfRuleFunc *)tmp_list->data;
+	for (tmp_list = conf->conf_add_rule_listener;
+			tmp_list; tmp_list = g_slist_next (tmp_list)) {
+		func = (SkkConfAddRuleFunc *)tmp_list->data;
 		if (func) {
 			func->listener (conf, key, hira, kata, append, func->user_data);
 		}
@@ -103,6 +114,24 @@ add_rule_emit (SkkConf *conf, const gchar *key, const gchar *hira,
 	return;
 }
 
+static void
+delete_rule_emit (SkkConf *conf, const gchar *key)
+{
+	GSList *tmp_list;
+	SkkConfDeleteRuleFunc *func;
+	if (!conf)
+		return;
+	for (tmp_list = conf->conf_delete_rule_listener;
+			tmp_list; tmp_list = g_slist_next (tmp_list)) {
+		func = (SkkConfDeleteRuleFunc *)tmp_list->data;
+		if (func) {
+			func->listener (conf, key, func->user_data);
+		}
+
+	}
+	return;
+}
+
 static gboolean
 find_item_func (gconstpointer p1, gconstpointer p2)
 {
@@ -144,6 +173,15 @@ find_item_for_delete (GList *list, SkkConfItem *item)
 	return NULL;
 }
 
+static void
+add_defaults (SkkConf* conf)
+{
+	gint i;
+	for (i = 0; i < SKKCONF_DEFAULT_ITEM_SIZE ; i++) {
+		skk_conf_add_item (conf, &itemp[i]);
+	}
+}
+
 static void
 add_item_bool (SkkConf *conf, SkkConfItem *item)
 {
@@ -319,6 +357,50 @@ err:
 	return;
 }
 
+static void
+parse_rule_delete (SkkConf *conf, xmlDocPtr doc)
+{
+	xmlXPathContextPtr context;
+	xmlXPathObjectPtr object;
+	xmlNodeSetPtr nodeset = NULL;
+	xmlNodePtr node = NULL;
+	xmlChar *key = NULL;
+	gint i = 0;
+	if (!doc)
+		return;
+	context = xmlXPathNewContext (doc);
+	if (!context)
+		return;
+	object = xmlXPathEval ("/iiimf-skk/delete-rule/key", context);
+	if (!object) {
+		xmlXPathFreeContext (context);
+		return;
+	}
+	if (object->type == XPATH_NODESET) {
+		nodeset = object->nodesetval;
+	}
+	/*
+	 * NOTICE:
+	 * nodesetval is not allocated by libxml2, so don't free after using it
+	 */
+	if (!nodeset || !nodeset->nodeTab) {
+		xmlXPathFreeObject (object);
+		return;
+	}
+	for (node = nodeset->nodeTab[0]; i < nodeset->nodeNr ; node = nodeset->nodeTab[i]) {
+		key = xmlNodeGetContent (node);
+		if (key) {
+			if (*key) 
+				delete_rule_emit (conf, key);
+			xmlFree (key);
+		}
+		i++;
+	}
+	xmlXPathFreeObject (object);
+	xmlXPathFreeContext (context);
+	return;
+}
+
 static void
 parse_item (SkkConf *conf, xmlDocPtr doc) 
 {
@@ -333,8 +415,10 @@ parse_item (SkkConf *conf, xmlDocPtr doc)
 	if (!context)
 		return;
 	object = xmlXPathEval ("/iiimf-skk/item", context);
-	if (!object)
+	if (!object) {
+		xmlXPathFreeContext (context);
 		return;
+	}
 	if (object->type == XPATH_NODESET) {
 		nodeset = object->nodesetval;
 	}
@@ -342,8 +426,10 @@ parse_item (SkkConf *conf, xmlDocPtr doc)
 	 * NOTICE:
 	 * nodesetval is not allocated by libxml2, so don't free after using it
 	 */
-	if (!nodeset || !nodeset->nodeTab)
+	if (!nodeset || !nodeset->nodeTab) {
+		xmlXPathFreeObject (object);
 		return;
+	}
 	for (node = nodeset->nodeTab[0]; i < nodeset->nodeNr ; node = nodeset->nodeTab[i]) {
 		parse_item_one (conf, node);
 		i++;
@@ -407,9 +493,11 @@ parse_rule_one (SkkConf *conf, xmlNodePtr node)
 	if (append)
 		xmlFree (append);
 	if (hira_e)
-		xmlFree (hira_e);
+		g_free (hira_e);
 	if (kata_e)
-		xmlFree (kata_e);
+		g_free (kata_e);
+	if (append_e)
+		g_free (append_e);
 	return;
 }
 
@@ -427,8 +515,10 @@ parse_rule (SkkConf *conf, xmlDocPtr doc)
 	if (!context)
 		return;
 	object = xmlXPathEval ("/iiimf-skk/rule", context);
-	if (!object)
+	if (!object) {
+		xmlXPathFreeContext (context);
 		return;
+	}
 	if (object->type == XPATH_NODESET) {
 		nodeset = object->nodesetval;
 	}
@@ -436,8 +526,10 @@ parse_rule (SkkConf *conf, xmlDocPtr doc)
 	 * NOTICE:
 	 * nodesetval is not allocated by libxml2, so don't free after using it
 	 */
-	if (!nodeset || !nodeset->nodeTab)
+	if (!nodeset || !nodeset->nodeTab) {
+		xmlXPathFreeObject (object);
 		return;
+	}
 	for (node = nodeset->nodeTab[0]; i < nodeset->nodeNr ; node = nodeset->nodeTab[i]) {
 		parse_rule_one (conf, node);
 		i++;
@@ -482,6 +574,11 @@ load_rc (SkkConf *conf, const char *filename)
 	if (stat (filename, &s) == -1)
 		return;
 
+	/* reset setting */
+	if (conf) {
+		clear (conf);
+		add_defaults (conf);
+	}
 	doc = xmlParseFile (filename);
 	if (!doc)
 		return;
@@ -490,6 +587,7 @@ load_rc (SkkConf *conf, const char *filename)
 		xmlFreeDoc (doc);
 		return;
 	}
+	parse_rule_delete (conf, doc);
 	parse_rule (conf, doc);
 	parse_item (conf, doc);
 	xmlFreeDoc(doc);
@@ -516,18 +614,16 @@ SkkConf*
 skk_conf_new (void)
 {
 	SkkConf *ret;
-	gint i;
 	ret = g_new (SkkConf, 1);
 	memset (ret, 0, sizeof (SkkConf));
 	ret->clear = clear;
 	ret->conf_string = NULL;
 	ret->conf_bool = NULL;
 	ret->conf_num = NULL;
-	ret->conf_rule_listener = NULL;
+	ret->conf_add_rule_listener = NULL;
+	ret->conf_delete_rule_listener = NULL;
 	/* initialize */
-	for (i = 0; i < SKKCONF_DEFAULT_ITEM_SIZE ; i++) {
-		skk_conf_add_item (ret, &itemp[i]);
-	}
+	add_defaults (ret);
 	skk_conf_ref (ret);
 	return ret;
 }
@@ -542,8 +638,11 @@ skk_conf_destroy (SkkConf *conf)
 		return;
 	}
 	skk_conf_clear (conf);
-	if (conf->conf_rule_listener) {
-		skk_utils_list_free (conf->conf_rule_listener, TRUE, NULL, NULL);
+	if (conf->conf_add_rule_listener) {
+		skk_utils_slist_free (conf->conf_add_rule_listener, TRUE, NULL, NULL);
+	}
+	if (conf->conf_delete_rule_listener) {
+		skk_utils_slist_free (conf->conf_delete_rule_listener, TRUE, NULL, NULL);
 	}
 	g_free (conf);
 }
@@ -555,7 +654,6 @@ skk_conf_clear (SkkConf *conf)
 		return;
 	if (conf->clear)
 		conf->clear (conf);
-	conf = NULL;
 	return;
 }
 
@@ -602,7 +700,7 @@ skk_conf_add_item (SkkConf *conf, SkkConfItem *item)
 	return;
 }
 
-SkkConfItem *
+static SkkConfItem *
 find_item (GList *list, SkkConfItemType type)
 {
 	GList *ret;
@@ -617,25 +715,75 @@ find_item (GList *list, SkkConfItemType type)
 void
 skk_conf_set_bool (SkkConf *conf, SkkConfItemType type, gboolean value)
 {
+	SkkConfItem *item;
 	/* TODO How to */
 	if (!conf)
 		return;
+	item = find_item (conf->conf_bool, type);
+	if (!item)
+		return;
+	item->value = GINT_TO_POINTER (value);
+	return;
+}
+
+void
+skk_conf_set_bool_by_name (SkkConf *conf, const gchar *name, gboolean value)
+{
+	if (!name)
+		return;
+	skk_conf_set_bool (conf, find_item_type (name), value);
 }
 
 void
 skk_conf_set_string (SkkConf *conf, SkkConfItemType type, const gchar *string)
 {
+	SkkConfItem *item;
+	gchar *value = NULL;
 	/* TODO How to */
 	if (!conf)
 		return;
+	if (!string)
+		return;
+	item = find_item (conf->conf_string, type);
+	if (!item)
+		return;
+	if (item->value) {
+		value = item->value;
+	}
+	item->value = g_strdup (string);
+	if (value)
+		g_free (value);
+	return;
+}
+
+void
+skk_conf_set_string_by_name (SkkConf *conf, const gchar *name, const gchar *string)
+{
+	if (!name)
+		return;
+	skk_conf_set_string (conf, find_item_type (name), string);
 }
 
 void
 skk_conf_set_num (SkkConf *conf, SkkConfItemType type, gint num)
 {
+	SkkConfItem *item;
 	/* TODO How to */
 	if (!conf)
 		return;
+	item = find_item (conf->conf_num, type);
+	if (!item)
+		return;
+	item->value = GINT_TO_POINTER (num);
+	return;
+}
+
+void
+skk_conf_set_num_by_name (SkkConf *conf, const gchar *name, gint num)
+{
+	if (!name)
+		return;
+	skk_conf_set_num (conf, find_item_type (name), num);
 }
 
 gboolean
@@ -772,17 +920,32 @@ skk_conf_item_destroy (SkkConfItem *item)
 }
 
 gboolean
-skk_conf_add_rule_listener (SkkConf *conf, SkkConfRuleListener listener, gpointer user_data)
+skk_conf_add_rule_listener (SkkConf *conf, SkkConfAddRuleListener listener, gpointer user_data)
 {
-	SkkConfRuleFunc *func;
+	SkkConfAddRuleFunc *func;
 	if (!conf)
 		return FALSE;
 	if (!listener)
 		return FALSE;
-	func = g_new0 (SkkConfRuleFunc, 1);
+	func = g_new0 (SkkConfAddRuleFunc, 1);
 	func->listener = listener;
 	func->user_data = user_data;
-	conf->conf_rule_listener = g_list_append (conf->conf_rule_listener, func);
+	conf->conf_add_rule_listener = g_slist_append (conf->conf_add_rule_listener, func);
+	return TRUE;
+}
+
+gboolean
+skk_conf_add_rule_delete_listener (SkkConf *conf, SkkConfDeleteRuleListener listener, gpointer user_data)
+{
+	SkkConfDeleteRuleFunc *func;
+	if (!conf)
+		return FALSE;
+	if (!listener)
+		return FALSE;
+	func = g_new0 (SkkConfDeleteRuleFunc, 1);
+	func->listener = listener;
+	func->user_data = user_data;
+	conf->conf_delete_rule_listener = g_slist_append (conf->conf_delete_rule_listener, func);
 	return TRUE;
 }
 
@@ -815,6 +978,10 @@ main (void)
 	printf("skk_egg_like_newline = %s\n",
 					skk_conf_get_bool_by_name (conf, "skk-egg-like-newline") ? "TRUE" : "FALSE");
 	printf("'%s' -> '%s'\n",".", skk_conv_get_hiragana(rule, ".", NULL));
+	skk_conf_set_bool (conf, SKKCONF_BOOL_EGG_LIKE_NEWLINE, FALSE);
+	printf("skk_egg_like_newline = %s\n",
+					skk_conf_get_bool_by_name (conf, "skk-egg-like-newline") ? "TRUE" : "FALSE");
+	skk_conf_set_bool_by_name (conf, "skk-egg-like-newline", TRUE);
 	printf("skk_egg_like_newline = %s\n",
 					skk_conf_get_bool_by_name (conf, "skk-egg-like-newline") ? "TRUE" : "FALSE");
 	printf ("skk_server_host = %s\n",
