@@ -32,12 +32,13 @@
  *
  */
 
-/* $Id: skkdictitem.c,v 1.3 2002/11/09 16:56:49 famao Exp $ */
+/* $Id: skkdictitem.c,v 1.1.2.4 2003/02/14 07:24:00 famao Exp $ */
 
 /* vi:set ts=4 sw=4: */
 
 #include <string.h>
 #include "skkdictitem.h"
+#include "skkdictutil.h"
 #include "skkutils.h"
 
 SkkDictItem*
@@ -55,8 +56,8 @@ skk_dict_item_new_with_value (const gchar *candidate, const gchar *annotation)
 {
 	SkkDictItem *ret;
 	ret = g_new0 (SkkDictItem, 1);
-	ret->candidate = candidate ? g_strdup (candidate) : NULL;
-	ret->annotation = annotation ? g_strdup (annotation) : NULL;
+	ret->candidate = candidate ? skk_dict_util_decode_string (candidate) : NULL;
+	ret->annotation = annotation ? skk_dict_util_decode_string (annotation) : NULL;
 	return ret;
 }
 
@@ -118,16 +119,24 @@ skk_dict_item_destroy_all (GList *list)
 gchar *
 skk_dict_item_to_string (SkkDictItem *item, const gchar *delim)
 {
+	gchar *encode_cand;
+	gchar *encode_anno;
+	gchar *ret;
 	if (!item)
 		return NULL;
 	if (!item->candidate)
 		return NULL;
+	encode_cand = skk_dict_util_encode_string (item->candidate);
 	/* when we have annotation, must define delim */
 	if (item->annotation && delim) {
-		return g_strconcat (item->candidate, delim, item->annotation, NULL);
+		encode_anno = skk_dict_util_encode_string (item->annotation);
+		ret = g_strconcat (encode_cand, delim, encode_anno, NULL);
+		g_free (encode_anno);
 	} else {
-		return g_strconcat (item->candidate, NULL);
+		ret = g_strconcat (encode_cand, NULL);
 	}
+	g_free (encode_cand);
+	return ret;
 }
 
 gchar *
