@@ -32,7 +32,7 @@
  *
  */
 
-/* $Id: skkldictimpl.c,v 1.5 2002/11/09 16:56:49 famao Exp $ */
+/* $Id: skkldictimpl.c,v 1.1.2.14 2002/12/27 18:44:57 famao Exp $ */
 
 /* vi:set ts=4 sw=4: */
 
@@ -47,7 +47,7 @@
 static SkkQueryFunctionImpl *get_new (SkkQuery *query);
 static GList *do_query (SkkQueryFunctionImpl *impl, const gchar *buf, const gchar *okuri, gint *found_num);
 static void destroy (SkkQueryFunctionImpl *impl);
-static const gchar* get_info (void);
+static gchar* get_info (void);
 static gint get_type (void);
 static GList *get_arg (void);
 static void set_arg (GList *arg);
@@ -297,7 +297,7 @@ get_type (void)
 	return SKKQUERY_LOCAL;
 }
 
-static const gchar *
+static gchar *
 get_info (void)
 {
 	return NULL;
@@ -307,16 +307,22 @@ static void
 destroy (SkkQueryFunctionImpl *impl)
 {
 	SkkLdictImpl *l = (SkkLdictImpl *)impl;
+#ifdef DB_HAVE_TRUNCATE
 	u_int32_t countp;
+#endif
 	/* TODO */
 	if (l->initialized) {
+#ifdef DB_HAVE_TRUNCATE
 		if (!l->file_ldict) {
 			l->ldict->truncate (l->ldict, NULL, &countp, 0);
 		}
+#endif
 		l->ldict->close (l->ldict, 0);
+#ifdef DB_HAVE_TRUNCATE
 		if (!l->file_ldict_okuri) {
 			l->ldict_okuri->truncate (l->ldict_okuri, NULL, &countp, 0);
 		}
+#endif
 		l->ldict_okuri->close (l->ldict_okuri, 0);
 		l->initialized = FALSE;
 	}
