@@ -32,7 +32,7 @@
  *
  */
 
-/* $Id: skkmode.c,v 1.5 2002/11/09 16:56:49 famao Exp $ */
+/* $Id: skkmode.c,v 1.4.2.5 2003/02/12 19:58:44 famao Exp $ */
 
 /* vi:set ts=4 sw=4: */
 
@@ -77,11 +77,11 @@ struct _SkkModeFunc {
 static void
 status_emit (SkkMode *mode)
 {
-	GList *tmp_list;
+	GSList *tmp_list;
 	SkkModeFunc *func;
 	if (!mode)
 		return;
-	for (tmp_list = mode->status_listener; tmp_list; tmp_list = g_list_next (tmp_list)) {
+	for (tmp_list = mode->status_listener; tmp_list; tmp_list = g_slist_next (tmp_list)) {
 		func = (SkkModeFunc *)tmp_list->data;
 		if (func) {
 			func->listener (mode, skk_mode_get_status (mode), func->user_data);
@@ -93,11 +93,11 @@ status_emit (SkkMode *mode)
 static void
 j_status_emit (SkkMode *mode)
 {
-	GList *tmp_list;
+	GSList *tmp_list;
 	SkkModeFunc *func;
 	if (!mode)
 		return;
-	for (tmp_list = mode->j_status_listener; tmp_list; tmp_list = g_list_next (tmp_list)) {
+	for (tmp_list = mode->j_status_listener; tmp_list; tmp_list = g_slist_next (tmp_list)) {
 		func = (SkkModeFunc *)tmp_list->data;
 		if (func) {
 			func->listener (mode, skk_mode_get_j_status (mode), func->user_data);
@@ -109,11 +109,11 @@ j_status_emit (SkkMode *mode)
 static void
 query_status_emit (SkkMode *mode)
 {
-	GList *tmp_list;
+	GSList *tmp_list;
 	SkkModeFunc *func;
 	if (!mode)
 		return;
-	for (tmp_list = mode->query_status_listener; tmp_list; tmp_list = g_list_next (tmp_list)) {
+	for (tmp_list = mode->query_status_listener; tmp_list; tmp_list = g_slist_next (tmp_list)) {
 		func = (SkkModeFunc *)tmp_list->data;
 		if (func)
 			func->listener (mode, skk_mode_get_query_status (mode), func->user_data);
@@ -345,15 +345,15 @@ skk_mode_destroy (SkkMode *mode)
 	if (!mode) return;
 	skk_mode_clear (mode);
 	if (mode->status_listener) {
-		skk_utils_list_free (mode->status_listener, TRUE, NULL, NULL);
+		skk_utils_slist_free (mode->status_listener, TRUE, NULL, NULL);
 		mode->status_listener = NULL;
 	}
 	if (mode->j_status_listener) {
-		skk_utils_list_free (mode->j_status_listener, TRUE, NULL, NULL);
+		skk_utils_slist_free (mode->j_status_listener, TRUE, NULL, NULL);
 		mode->j_status_listener = NULL;
 	}
 	if (mode->query_status_listener) {
-		skk_utils_list_free (mode->query_status_listener, TRUE, NULL, NULL);
+		skk_utils_slist_free (mode->query_status_listener, TRUE, NULL, NULL);
 		mode->query_status_listener = NULL;
 	}
 	if (mode->latin_str)
@@ -556,7 +556,7 @@ skk_mode_add_status_listener (SkkMode *mode, SkkModeListener listener, gpointer
 	func = g_new (SkkModeFunc, 1);
 	func->listener =listener;
 	func->user_data = user_data;
-	mode->status_listener = g_list_append (mode->status_listener, func);
+	mode->status_listener = g_slist_append (mode->status_listener, func);
 }
 
 void
@@ -570,7 +570,7 @@ skk_mode_add_j_status_listener (SkkMode *mode, SkkModeListener listener, gpointe
 	func = g_new (SkkModeFunc, 1);
 	func->listener = listener;
 	func->user_data = user_data;
-	mode->j_status_listener = g_list_append (mode->j_status_listener, func);
+	mode->j_status_listener = g_slist_append (mode->j_status_listener, func);
 	return;
 }
 
@@ -585,7 +585,7 @@ skk_mode_add_query_status_listener (SkkMode *mode, SkkModeListener listener, gpo
 	func = g_new (SkkModeFunc, 1);
 	func->listener = listener;
 	func->user_data = user_data;
-	mode->query_status_listener = g_list_append (mode->query_status_listener, func);
+	mode->query_status_listener = g_slist_append (mode->query_status_listener, func);
 	return;
 }
 
@@ -594,9 +594,14 @@ skk_mode_set_latin_str (SkkMode *mode, const gchar *str)
 {
 	if (!mode || !str)
 		return;
-	if (mode->latin_str)
+	if (mode->latin_str) {
+		if (!strcmp (mode->latin_str, str)) {
+			return;
+		}
 		g_free (mode->latin_str);
+	}
 	mode->latin_str = g_strdup (str);
+	status_emit (mode);
 	return;
 }
 
@@ -605,9 +610,14 @@ skk_mode_set_hirakana_str (SkkMode *mode, const gchar *str)
 {
 	if (!mode || !str)
 		return;
-	if (mode->hirakana_str)
+	if (mode->hirakana_str) {
+		if (!strcmp (mode->hirakana_str, str)) {
+			return;
+		}
 		g_free (mode->hirakana_str);
+	}
 	mode->hirakana_str = g_strdup (str);
+	status_emit (mode);
 	return;
 }
 
@@ -616,9 +626,14 @@ skk_mode_set_katakana_str (SkkMode *mode, const gchar *str)
 {
 	if (!mode || !str)
 		return;
-	if (mode->katakana_str)
+	if (mode->katakana_str) {
+		if (!strcmp (mode->katakana_str, str)) {
+			return;
+		}
 		g_free (mode->katakana_str);
+	}
 	mode->katakana_str = g_strdup (str);
+	status_emit (mode);
 	return;
 }
 
@@ -627,9 +642,14 @@ skk_mode_set_jisx0208_latin_str (SkkMode *mode, const gchar *str)
 {
 	if (!mode || !str)
 		return;
-	if (mode->jisx0208_latin_str)
+	if (mode->jisx0208_latin_str) {
+		if (!strcmp (mode->jisx0208_latin_str, str)) {
+			return;
+		}
 		g_free (mode->jisx0208_latin_str);
+	}
 	mode->jisx0208_latin_str = g_strdup (str);
+	status_emit (mode);
 	return;
 }
 
@@ -638,9 +658,14 @@ skk_mode_set_jisx0201_kana_str (SkkMode *mode, const gchar *str)
 {
 	if (!mode || !str)
 		return;
-	if (mode->jisx0201_kana_str)
+	if (mode->jisx0201_kana_str) {
+		if (!strcmp (mode->jisx0201_kana_str, str)) {
+			return;
+		}
 		g_free (mode->jisx0201_kana_str);
+	}
 	mode->jisx0201_kana_str = g_strdup (str);
+	status_emit (mode);
 	return;
 }
 
@@ -649,9 +674,14 @@ skk_mode_set_abbrev_str (SkkMode *mode, const gchar *str)
 {
 	if (!mode || !str)
 		return;
-	if (mode->abbrev_str)
+	if (mode->abbrev_str) {
+		if (!strcmp (mode->abbrev_str, str)) { /* do nothing */
+			return;
+		}
 		g_free (mode->abbrev_str);
+	}
 	mode->abbrev_str = g_strdup (str);
+	status_emit (mode);
 	return;
 }
 
