@@ -1,7 +1,8 @@
 /*
  * IIIMF-SKK, Japanese Language Engine for 
  *                        IIIMF (Internet/Intranet Input Method Framework)
- * 
+ *
+ * Copyright (C) 2003 Motonobu Ichimura <famao@momonga-linux.org>
  * Copyright (C) 2002 Motonobu Ichimura <famao@kondara.org>
  *
  * All rights reserved.
@@ -33,7 +34,7 @@
  *
  */
 
-/* $Id: skk.c,v 1.14 2002/11/09 16:56:50 famao Exp $ */
+/* $Id: skk.c,v 1.9.2.27 2003/02/12 22:09:21 famao Exp $ */
 
 /* vi:set ts=4 sw=4: */
 
@@ -254,6 +255,7 @@ struct _SkkSessionData
 
 struct _SkkDesktopData
 {
+	int drop_priv;
 	SkkFunc *skkfunc;
 	SkkQuery *skkquery;
 	SkkConf *skkconf;
@@ -451,7 +453,7 @@ if_GetIfInfo (IMArgList args, int num_args)
 					objects[obj_id].type = IM_DOWNLOADINGOBJECT_BINGUI_TYPE;
 					objects[obj_id].name = AuxName;
 					objects[obj_id].name_length = 3;
-					objects[obj_id].domain = "org.kondara";
+					objects[obj_id].domain = "org.momonga-linux";
 					objects[obj_id].scope = "skk";
 					objects[obj_id].path = "./locale/ja/skk/xaux/xaux.so";
 					objects[obj_id].signature = "";
@@ -513,7 +515,6 @@ static Bool
 if_skk_OpenDesktop (iml_desktop_t * desktop, IMArgList args, int num_args)
 {
 	int i;
-	Bool drop_priv;
 	SkkDesktopData *d;
 
 #ifdef DEBUG
@@ -578,14 +579,14 @@ if_skk_OpenDesktop (iml_desktop_t * desktop, IMArgList args, int num_args)
 				break;
 		}
 	}
-	drop_priv = skk_drop_privilege (desktop->user_name);
 	d = g_new0 (SkkDesktopData, 1);
+	d->drop_priv = skk_drop_privilege (desktop->user_name);
 	d->skkfunc = skk_func_new ();
 	d->skkquery = skk_query_new ();
 	d->skkconf = skk_conf_new ();
 	d->skkconv = skk_conv_rule_new ();
 	skk_conf_add_rule_listener (d->skkconf, skk_add_rule_listener, d->skkconv);
-	if (drop_priv) {
+	if (d->drop_priv) {
 #ifdef DEBUG
 		printf ("Drop Privilege\n");
 #endif
@@ -664,6 +665,9 @@ skk_add_rule_listener (SkkConf *conf , const gchar *key,
 		const gchar *hira, const gchar *kata, const gchar *append, gpointer user_data)
 {
 	SkkConvRule *rule = (SkkConvRule *)user_data;
+#if DEBUG
+	printf ("add rule %s %s %s %s\n", key, hira, kata, append);
+#endif
 	skk_conv_add_rule (rule, skk_conv_rule_item_new_with_value (key, hira, kata, append));
 	return;
 }
@@ -1232,7 +1236,8 @@ trans:
 			candidate = skk_buffer_get_nth_candidate (buf, buf->candidate_count + num);
 			if (candidate) {
 				skk_commit_string (s, candidate);
-				skk_buffer_add_dict_with_value (buf, buf->has_num ? skkbuf_numb (buf) : candidate);
+				skk_buffer_add_dict_with_value (buf, buf->has_num ? skkbuf_numb (buf) :
+						skkbuf_nth_candidate (buf, buf->candidate_count + num));
 				skk_buffer_clear (buf);
 				g_free (candidate);
 			}
@@ -2127,6 +2132,7 @@ static void
 skk_process_auxevent (iml_session_t *s, IMAuxDrawCallbackStruct *aux)
 {
 	int i;
+	SkkDesktopData *d = skk_desktop_data (s);
 	SkkSessionData *ses = skk_session_data (s);
 	SkkBuffer *buf = skk_session_buf (s);
 	for (i = 0; i < aux->count_integer_values; i++) {
@@ -2198,6 +2204,17 @@ skk_process_auxevent (iml_session_t *s, IMAuxDrawCallbackStruct *aux)
 				DEBUG_DO (printf ("dict initialized\n"));
 				ses->dict_aux_ok = True;
 				break;
+			case SKK_AUX_RELOAD_CONFIG:
+				/* famao */
+
+				if (d->drop_priv) {
+					/* reset conv_rule */
+					skk_conv_rule_clear (d->skkconv);
+					/* skk_conf_clear (d->skkconf); */
+					skk_conf_load_rc (d->skkconf, s->desktop->user_name);
+				}
+				break;
+
 			default:
 				break;
 		}
